@@ -1,8 +1,8 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * Almanah
- * Copyright (C) Philip Withnall 2008 <philip@tecnocode.co.uk>
- * 
+ * Copyright (C) Philip Withnall 2008-2009 <philip@tecnocode.co.uk>
+ *
  * Almanah is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -18,6 +18,7 @@
  */
 
 #include <config.h>
+#include <time.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -30,14 +31,17 @@
 #include "main.h"
 #include "interface.h"
 #include "add-definition-dialog.h"
+#include "preferences-dialog.h"
+#include "search-dialog.h"
+#include "date-entry-dialog.h"
 #include "printing.h"
 #include "entry.h"
 #include "storage-manager.h"
 #include "event.h"
 #include "definition.h"
 #include "definition-manager-window.h"
+#include "import-dialog.h"
 
-static void almanah_main_window_init (AlmanahMainWindow *self);
 static void almanah_main_window_dispose (GObject *object);
 static void save_window_state (AlmanahMainWindow *self);
 static void restore_window_state (AlmanahMainWindow *self);
@@ -51,11 +55,35 @@ static void mw_bold_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main
 static void mw_italic_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
 static void mw_underline_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
 static void mw_events_updated_cb (AlmanahEventManager *event_manager, AlmanahEventFactoryType type_id, AlmanahMainWindow *main_window);
-void mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_window);
 static void mw_events_selection_changed_cb (GtkTreeSelection *tree_selection, AlmanahMainWindow *main_window);
 static void mw_events_value_data_cb (GtkTreeViewColumn *column, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data);
 static void mw_definition_removed_cb (AlmanahStorageManager *storage_manager, const gchar *definition_text, AlmanahMainWindow *main_window);
 
+/* GtkBuilder callbacks */
+void mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_window);
+void mw_import_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_page_setup_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_print_preview_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_print_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_quit_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_cut_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_copy_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_paste_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_delete_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_insert_time_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_important_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_select_date_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_search_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_preferences_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_jump_to_today_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_add_definition_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_remove_definition_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_view_definitions_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
+void mw_events_tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, AlmanahMainWindow *main_window);
+gboolean mw_entry_view_focus_out_event_cb (GtkWidget *entry_view, GdkEventFocus *event, AlmanahMainWindow *main_window);
+void mw_view_button_clicked_cb (GtkButton *button, AlmanahMainWindow *main_window);
+
 struct _AlmanahMainWindowPrivate {
 	GtkTextView *entry_view;
 	GtkTextBuffer *entry_buffer;
@@ -74,6 +102,7 @@ struct _AlmanahMainWindowPrivate {
 	GtkAction *cut_action;
 	GtkAction *copy_action;
 	GtkAction *delete_action;
+	GtkAction *important_action;
 
 	gboolean updating_formatting;
 	gboolean pending_bold_active;
@@ -180,9 +209,7 @@ almanah_main_window_new (void)
 	priv->cut_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_cut"));
 	priv->copy_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_copy"));
 	priv->delete_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_delete"));
-
-	/* Set up text formatting */
-	almanah_interface_create_text_tags (priv->entry_buffer, TRUE);
+	priv->important_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_important"));
 
 #ifdef ENABLE_SPELL_CHECKING
 	/* Set up spell checking, if it's enabled */
@@ -190,6 +217,10 @@ almanah_main_window_new (void)
 		almanah_main_window_enable_spell_checking (main_window, NULL);
 #endif /* ENABLE_SPELL_CHECKING */
 
+	/* Set up text formatting. It's important this is done after setting up GtkSpell, so that we know whether to
+	 * create a dummy gtkspell-misspelled text tag. */
+	almanah_interface_create_text_tags (priv->entry_buffer, TRUE);
+
 	/* Make sure we're notified if the cursor moves position so we can check the tag stack */
 	g_signal_connect (priv->entry_buffer, "notify::cursor-position", G_CALLBACK (mw_entry_buffer_cursor_position_cb), main_window);
 
@@ -223,10 +254,6 @@ almanah_main_window_new (void)
 	g_signal_connect (priv->events_selection, "changed", G_CALLBACK (mw_events_selection_changed_cb), main_window);
 	gtk_tree_view_column_set_cell_data_func (priv->event_value_column, GTK_CELL_RENDERER (priv->event_value_renderer), mw_events_value_data_cb, NULL, NULL);
 
-	/* Prettify the UI */
-	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "almanah_mw_calendar_label")));
-	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "almanah_mw_events_label")));
-
 #ifndef ENABLE_ENCRYPTION
 #ifndef ENABLE_SPELL_CHECKING
 	/* Remove the "Preferences" entry from the menu */
@@ -340,7 +367,8 @@ save_current_entry (AlmanahMainWindow *self)
 		gchar date_string[100];
 		GtkWidget *dialog;
 
-		g_date_strftime (date_string, sizeof (date_string), "%A, %e %B %Y", &date);
+		/* Translators: This is a strftime()-format string for the date to display when asking about editing a diary entry. */
+		g_date_strftime (date_string, sizeof (date_string), _("%A, %e %B %Y"), &date);
 
 		dialog = gtk_message_dialog_new (GTK_WINDOW (self),
 						 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
@@ -364,7 +392,8 @@ save_current_entry (AlmanahMainWindow *self)
 		gchar date_string[100];
 		GtkWidget *dialog;
 
-		g_date_strftime (date_string, sizeof (date_string), "%A, %e %B %Y", &date);
+		/* Translators: This is a strftime()-format string for the date to display when asking about deleting a diary entry. */
+		g_date_strftime (date_string, sizeof (date_string), _("%A, %e %B %Y"), &date);
 
 		dialog = gtk_message_dialog_new (GTK_WINDOW (self),
 						 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
@@ -416,6 +445,7 @@ add_definition_to_current_entry (AlmanahMainWindow *self)
 	AlmanahMainWindowPrivate *priv = self->priv;
 	GtkTextIter start_iter, end_iter;
 	gchar *text;
+	AlmanahDefinition *definition;
 
 	g_assert (priv->entry_buffer != NULL);
 	g_assert (gtk_text_buffer_get_char_count (priv->entry_buffer) != 0);
@@ -425,6 +455,19 @@ add_definition_to_current_entry (AlmanahMainWindow *self)
 
 	text = gtk_text_buffer_get_text (priv->entry_buffer, &start_iter, &end_iter, FALSE);
 
+	/* If the definition already exists, don't display the dialogue */
+	definition = almanah_storage_manager_get_definition (almanah->storage_manager, text);
+	if (definition != NULL) {
+		g_object_unref (definition);
+
+		/* Add a GtkTextTag to the GtkTextBuffer to mark the definition */
+		gtk_text_buffer_apply_tag_by_name (priv->entry_buffer, "definition", &start_iter, &end_iter);
+		gtk_text_buffer_set_modified (priv->entry_buffer, TRUE);
+
+		g_free (text);
+		return;
+	}
+
 	/* Create the Add Definition dialogue if it doesn't already exist */
 	if (almanah->add_definition_dialog == NULL)
 		almanah->add_definition_dialog = GTK_WIDGET (almanah_add_definition_dialog_new ());
@@ -435,8 +478,6 @@ add_definition_to_current_entry (AlmanahMainWindow *self)
 	gtk_widget_show_all (almanah->add_definition_dialog);
 
 	if (gtk_dialog_run (GTK_DIALOG (almanah->add_definition_dialog)) == GTK_RESPONSE_OK) {
-		AlmanahDefinition *definition;
-
 		definition = almanah_add_definition_dialog_get_definition (ALMANAH_ADD_DEFINITION_DIALOG (almanah->add_definition_dialog));
 		if (definition == NULL)
 			return;
@@ -483,7 +524,7 @@ remove_definition_from_current_entry (AlmanahMainWindow *self)
 		end_iter = start_iter;
 		gtk_text_iter_forward_to_tag_toggle (&end_iter, tag);
 	}
-		
+
 	/* Remove the tag */
 	gtk_text_buffer_remove_tag (priv->entry_buffer, tag, &start_iter, &end_iter);
 	gtk_text_buffer_set_modified (priv->entry_buffer, TRUE);
@@ -551,7 +592,7 @@ mw_entry_buffer_cursor_position_cb (GObject *object, GParamSpec *pspec, AlmanahM
 		} else if (action != NULL) {
 			/* Force the toggle status on the action */
 			gtk_toggle_action_set_active (action, TRUE);
-		} else {
+		} else if (strcmp (tag_name, "gtkspell-misspelled") != 0) {
 			/* Print a warning about the unknown tag */
 			g_warning (_("Unknown or duplicate text tag \"%s\" in entry. Ignoring."), tag_name);
 		}
@@ -657,19 +698,29 @@ mw_delete_event_cb (GtkWindow *window, gpointer user_data)
 }
 
 void
-mw_page_setup_activate_cb (GtkAction *action, gpointer user_data)
+mw_import_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+{
+	if (almanah->import_dialog == NULL)
+		almanah->import_dialog = GTK_WIDGET (almanah_import_dialog_new ());
+
+	gtk_widget_show_all (almanah->import_dialog);
+	gtk_dialog_run (GTK_DIALOG (almanah->import_dialog));
+}
+
+void
+mw_page_setup_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 	almanah_print_page_setup ();
 }
 
 void
-mw_print_preview_activate_cb (GtkAction *action, gpointer user_data)
+mw_print_preview_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 	almanah_print_entries (TRUE);
 }
 
 void
-mw_print_activate_cb (GtkAction *action, gpointer user_data)
+mw_print_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 	almanah_print_entries (FALSE);
 }
@@ -709,7 +760,40 @@ mw_delete_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 }
 
 void
-mw_search_activate_cb (GtkAction *action, gpointer user_data)
+mw_insert_time_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+{
+	gchar time_string[100];
+	time_t time_struct;
+
+	time_struct = time (NULL);
+	strftime (time_string, sizeof (time_string), "%X", localtime (&time_struct));
+	gtk_text_buffer_insert_at_cursor (main_window->priv->entry_buffer, time_string, -1);
+}
+
+void
+mw_important_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+{
+	almanah_entry_set_is_important (main_window->priv->current_entry, gtk_toggle_action_get_active(GTK_TOGGLE_ACTION (action)));
+}
+
+void
+mw_select_date_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+{
+	if (almanah->date_entry_dialog == NULL)
+		almanah->date_entry_dialog = GTK_WIDGET (almanah_date_entry_dialog_new ());
+
+	gtk_widget_show_all (almanah->date_entry_dialog);
+	if (almanah_date_entry_dialog_run (ALMANAH_DATE_ENTRY_DIALOG (almanah->date_entry_dialog)) == TRUE) {
+		GDate new_date;
+
+		/* Switch to the specified date */
+		almanah_date_entry_dialog_get_date (ALMANAH_DATE_ENTRY_DIALOG (almanah->date_entry_dialog), &new_date);
+		almanah_main_window_select_date (main_window, &new_date);
+	}
+}
+
+void
+mw_search_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 	if (almanah->search_dialog == NULL)
 		almanah->search_dialog = GTK_WIDGET (almanah_search_dialog_new ());
@@ -719,7 +803,7 @@ mw_search_activate_cb (GtkAction *action, gpointer user_data)
 }
 
 void
-mw_preferences_activate_cb (GtkAction *action, gpointer user_data)
+mw_preferences_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 #ifdef ENABLE_ENCRYPTION
 	if (almanah->preferences_dialog == NULL)
@@ -804,7 +888,7 @@ mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 
 	gtk_show_about_dialog (GTK_WINDOW (main_window),
 				"version", VERSION,
-				"copyright", _("Copyright \xc2\xa9 2008 Philip Withnall"),
+				"copyright", _("Copyright \xc2\xa9 2008-2009 Philip Withnall"),
 				"comments", description,
 				"authors", authors,
 				/* Translators: please include your names here to be credited for your hard work!
@@ -817,7 +901,7 @@ mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 				"license", license,
 				"wrap-license", TRUE,
 				"website-label", _("Almanah Website"),
-				"website", "http://tecnocode.co.uk/projects/almanah",
+				"website", "http://live.gnome.org/Almanah_Diary",
 				NULL);
 
 	g_free (license);
@@ -944,7 +1028,6 @@ mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_wind
 	/* Translators: This is a strftime()-format string for the date displayed at the top of the main window. */
 	g_date_strftime (calendar_string, sizeof (calendar_string), _("%A, %e %B %Y"), &calendar_date);
 	gtk_label_set_markup (priv->date_label, calendar_string);
-	almanah_interface_embolden_label (priv->date_label);
 
 	/* Update the entry */
 	if (priv->current_entry != NULL)
@@ -955,6 +1038,8 @@ mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_wind
 
 	gtk_text_view_set_editable (priv->entry_view, almanah_entry_get_editability (priv->current_entry) != ALMANAH_ENTRY_FUTURE ? TRUE : FALSE);
 	gtk_text_buffer_set_modified (priv->entry_buffer, FALSE);
+	gtk_action_set_sensitive (priv->important_action, almanah_entry_get_editability (priv->current_entry) != ALMANAH_ENTRY_FUTURE ? TRUE : FALSE);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (priv->important_action), almanah_entry_is_important (priv->current_entry));
 
 	/* Prepare for the possibility of failure --- do as much of the general interface changes as possible first */
 	gtk_list_store_clear (priv->event_store);
@@ -1082,7 +1167,20 @@ almanah_main_window_enable_spell_checking (AlmanahMainWindow *self, GError **err
 {
 	GtkSpell *gtkspell;
 	gchar *spelling_language;
+	GtkTextTagTable *table;
+	GtkTextTag *tag;
 
+	/* Bail out if spell checking's already enabled */
+	if (gtkspell_get_from_text_view (self->priv->entry_view) != NULL)
+		return TRUE;
+
+	/* If spell checking wasn't already enabled, we have a dummy gtkspell-misspelled text tag to destroy */
+	table = gtk_text_buffer_get_tag_table (self->priv->entry_buffer);
+	tag = gtk_text_tag_table_lookup (table, "gtkspell-misspelled");
+	if (tag != NULL)
+		gtk_text_tag_table_remove (table, tag);
+
+	/* Get the spell checking language */
 	spelling_language = gconf_client_get_string (almanah->gconf_client, "/apps/almanah/spelling_language", NULL);
 
 	/* Make sure it's either NULL or a proper locale specifier */
@@ -1103,11 +1201,20 @@ void
 almanah_main_window_disable_spell_checking (AlmanahMainWindow *self)
 {
 	GtkSpell *gtkspell;
+	GtkTextTagTable *table;
+	GtkTextTag *tag;
 
 	gtkspell = gtkspell_get_from_text_view (self->priv->entry_view);
-	if (gtkspell != NULL) {
+	if (gtkspell != NULL)
 		gtkspell_detach (gtkspell);
-		g_object_unref (gtkspell);
-	}
+
+	/* Remove the old gtkspell-misspelling text tag */
+	table = gtk_text_buffer_get_tag_table (self->priv->entry_buffer);
+	tag = gtk_text_tag_table_lookup (table, "gtkspell-misspelled");
+	if (tag != NULL)
+		gtk_text_tag_table_remove (table, tag);
+
+	/* Create a dummy gtkspell-misspelling text tag */
+	gtk_text_buffer_create_tag (self->priv->entry_buffer, "gtkspell-misspelled", NULL);
 }
 #endif /* ENABLE_SPELL_CHECKING */
